﻿using System;
using Ozeki.MediaGateway;

namespace Web2Web_Voice_Server
{
    /// <summary>
    /// Represents a connected client.
    /// </summary>
    class MyClient
    {
        /// <summary>
        /// Client reference.
        /// </summary>
        IClient _client;

        /// <summary>
        /// Indicates the client is in conversation or not.
        /// </summary>
        public bool IsBusy { get; set; }

        /// <summary>
        /// The name of the client.
        /// </summary>
        public string Name { get; private set; }

        /// <summary>
        /// The reference of the remoteparty during communication.
        /// </summary>
        public MyClient RemoteParty { get; set; }


        /// <summary>
        /// Creaates a new instance of MyClient object.
        /// </summary>
        /// <param name="client">The connected client reference.</param>
        /// <param name="name">Name of the client.</param>
        public MyClient(IClient client, string name)
        {
            _client = client;
            Name = name;
        }

        #region Client-side method invoke

        /// <summary>
        /// Invokes client's callback method, to indicates for the client he can start to play the given stream.
        /// </summary>
        /// <param name="remotpartyId"> The id of the playable stream.</param>
        public void OnStartPlay(string remotpartyId)
        {
            _client.InvokeMethod("OnPlay", remotpartyId);
        }

        /// <summary>
        /// Invokes client's callback method. Notifies the client that it is ready to start or receive a call.
        /// </summary>
        public void OnSetReadyStatus(bool isReady, string name)
        {
            try
            {
                _client.InvokeMethod("OnSetReadyStatus", isReady, name);
            }
            catch (Exception)
            {}    
        }

        /// <summary>
        /// Invokes client's callback method. Notifies the client that it received a new call from another extension.
        /// </summary>
        /// <param name="requestOwner">The call request sender's clientId.</param>
        public void OnCallRequest(string requestOwner)
        {
            Console.WriteLine("Call request received from {0} to {1}",requestOwner, Name);
            RemoteParty.IsBusy = true;
            IsBusy = true;
            _client.InvokeMethod("OnCallRequest", requestOwner);
        }

        /// <summary>
        /// Invoke client's callback method. Sends the publish and receive stream ID.
        /// </summary>
        public void OnInCall()
        {
            Console.WriteLine("Sends 'start publishing' sign to the clients.");
            _client.InvokeMethod("OnInCall");
            RemoteParty._client.InvokeMethod("OnInCall");
        }

        /// <summary>
        /// Indicates for the client he can play the given published stream.
        /// </summary>
        public void OnPlayRemoteStream()
        {
            Console.WriteLine("PlayRemoteStream - client Name : {0} starts to play remoteStream: {1}", RemoteParty.Name, Name);
            RemoteParty._client.InvokeMethod("OnPlayRemoteStream", Name);
        }

        /// <summary>
        /// Invokes client's callback method, for indicates the other party hanged up the call.
        /// </summary>
        public void OnCallStop()
        {
            IsBusy = false;
            RemoteParty.IsBusy = false;
            _client.InvokeMethod("OnCallStop");
        }

        #endregion
    }
}
